@@ -9,7 +9,7 @@ import pyarrow as pa
 from tests.conftest import write_sidecar_route
 from webbed_duck.config import load_config
 from webbed_duck.core.compiler import compile_routes
-from webbed_duck.core.routes import load_compiled_routes
+from webbed_duck.core.routes import RouteDefinition, load_compiled_routes
 from webbed_duck.server.app import create_app
 from webbed_duck.server import cache as cache_mod
 
@@ -300,6 +300,87 @@ def test_invariant_filter_case_insensitive_handles_large_string() -> None:
     assert filtered.column("product_code").to_pylist() == ["Widget", "widget"]
 
 
+@pytest.mark.skipif(TestClient is None, reason="fastapi is not available")
+def test_invariant_filter_supports_null_requests(
+    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
+) -> None:
+    src = tmp_path / "src"
+    src.mkdir()
+    build = tmp_path / "build"
+    route_text = (
+        "+++\n"
+        "id = \"inventory_nulls\"\n"
+        "path = \"/inventory_nulls\"\n"
+        "title = \"Inventory nulls\"\n"
+        "[params.product_code]\n"
+        "type = \"str\"\n"
+        "required = false\n"
+        "[cache]\n"
+        "rows_per_page = 5\n"
+        "invariant_filters = [ { param = \"product_code\", column = \"product_code\" } ]\n"
+        "order_by = [\"seq\"]\n"
+        "+++\n\n"
+        "```sql\n"
+        "SELECT product_code, seq\n"
+        "FROM (VALUES\n"
+        "    ('widget', 1),\n"
+        "    (NULL, 2),\n"
+        "    ('gadget', 3)\n"
+        ") AS inventory(product_code, seq)\n"
+        "WHERE product_code IS NOT DISTINCT FROM COALESCE(NULLIF({{ product_code }}, ''), product_code)\n"
+        "ORDER BY seq\n"
+        "```\n"
+    )
+    write_sidecar_route(src, "inventory_nulls", route_text)
+    compile_routes(src, build)
+    routes = load_compiled_routes(build)
+    config = load_config(None)
+    config.server.storage_root = tmp_path / "storage"
+    config.server.storage_root.mkdir()
+    app = create_app(routes, config)
+    client = TestClient(app)
+
+    real_connect = duckdb.connect
+    calls: list[tuple[tuple[object, ...], dict[str, object]]] = []
+
+    def counting_connect(*args, **kwargs):
+        calls.append((args, kwargs))
+        return real_connect(*args, **kwargs)
+
+    monkeypatch.setattr(duckdb, "connect", counting_connect)
+
+    superset = client.get("/inventory_nulls", params={"format": "json"})
+    assert superset.status_code == 200
+    assert len(calls) == 1
+
+    payload = superset.json()
+    assert [row["product_code"] for row in payload["rows"]] == ["widget", None, "gadget"]
+
+    no_filter = client.get(
+        "/inventory_nulls",
+        params=[("format", "json"), ("product_code", "")],
+    )
+    assert no_filter.status_code == 200
+    assert no_filter.json()["rows"] == payload["rows"]
+
+    explicit_null = client.get(
+        "/inventory_nulls",
+        params=[("format", "json"), ("product_code", "__null__")],
+    )
+    assert explicit_null.status_code == 200
+    assert len(calls) == 2
+    explicit_rows = explicit_null.json()["rows"]
+    assert explicit_rows == [{"product_code": None, "seq": 2}]
+
+    repeated_null = client.get(
+        "/inventory_nulls",
+        params=[("format", "json"), ("product_code", "__null__")],
+    )
+    assert repeated_null.status_code == 200
+    assert len(calls) == 2
+    assert repeated_null.json()["rows"] == explicit_rows
+
+
 @pytest.mark.skipif(TestClient is None, reason="fastapi is not available")
 def test_invariant_combines_filtered_caches(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
     src = tmp_path / "src"
@@ -680,6 +761,118 @@ def test_invariant_unique_values_respect_filter_context(tmp_path: Path) -> None:
     assert "Assembly Line 1" not in department_block
 
 
+def test_normalize_invariant_value_interprets_null_tokens() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="code", column="code")
+
+    assert cache_mod.normalize_invariant_value("", setting) == []
+    assert cache_mod.normalize_invariant_value("__null__", setting) == [None]
+    assert cache_mod.normalize_invariant_value("__NULL__", setting) == [None]
+    assert cache_mod.normalize_invariant_value(["", "widget"], setting) == ["widget"]
+
+    multi_setting = cache_mod.InvariantFilterSetting(param="code", column="code", separator=",")
+    assert cache_mod.normalize_invariant_value("widget, __null__ , gadget", multi_setting) == [
+        "widget",
+        None,
+        "gadget",
+    ]
+
+
+def test_canonicalize_invariant_value_normalizes_decimals() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="amount", column="amount")
+
+    token = cache_mod.canonicalize_invariant_value(Decimal("1.00"), setting)
+    assert token.startswith("num:")
+    assert token == cache_mod.canonicalize_invariant_value(Decimal("1.0"), setting)
+    assert token == cache_mod.canonicalize_invariant_value(Decimal("1"), setting)
+
+
+def test_canonicalize_invariant_value_aligns_numeric_types() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="amount", column="amount")
+
+    base = cache_mod.canonicalize_invariant_value(1, setting)
+
+    assert base == cache_mod.canonicalize_invariant_value(1.0, setting)
+    assert base == cache_mod.canonicalize_invariant_value(Decimal("1.0"), setting)
+
+
+def test_canonicalize_invariant_value_normalizes_negative_zero() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="amount", column="amount")
+
+    assert cache_mod.canonicalize_invariant_value(-0.0, setting) == cache_mod.canonicalize_invariant_value(
+        0, setting
+    )
+    assert cache_mod.canonicalize_invariant_value(-0.0, setting) == cache_mod.canonicalize_invariant_value(
+        Decimal("0"), setting
+    )
+
+
+def test_canonicalize_invariant_mapping_collapses_decimal_variants() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="amount", column="amount")
+    mapping = cache_mod.canonicalize_invariant_mapping(
+        {"amount": (Decimal("2.500"), Decimal("2.50"))},
+        [setting],
+    )
+
+    assert mapping == {"amount": [cache_mod.canonicalize_invariant_value(Decimal("2.5"), setting)]}
+
+
+def test_canonicalize_invariant_mapping_collapses_numeric_variants() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="amount", column="amount")
+
+    mapping = cache_mod.canonicalize_invariant_mapping(
+        {"amount": (1, 1.0, Decimal("1.00"), Decimal("1"), -0.0, 0)},
+        [setting],
+    )
+
+    expected_tokens = {
+        cache_mod.canonicalize_invariant_value(Decimal("1"), setting),
+        cache_mod.canonicalize_invariant_value(Decimal("0"), setting),
+    }
+
+    assert mapping == {"amount": sorted(expected_tokens)}
+
+
+def test_cache_key_ignores_invariant_order(tmp_path: Path) -> None:
+    store = cache_mod.CacheStore(tmp_path)
+    route = RouteDefinition(
+        id="inventory",
+        path="/inventory",
+        methods=("GET",),
+        raw_sql="SELECT 1",
+        prepared_sql="SELECT 1",
+        param_order=("product_code",),
+        params=(),
+        metadata={},
+        version="v1",
+    )
+
+    first = cache_mod.InvariantFilterSetting(param="product_code", column="product_code")
+    second = cache_mod.InvariantFilterSetting(param="region", column="region", case_insensitive=True)
+
+    params = {"product_code": "widget", "region": "EMEA"}
+
+    base_settings = dict(
+        enabled=True,
+        ttl_seconds=3600,
+        rows_per_page=10,
+        enforce_page_size=True,
+        order_by=("product_code",),
+    )
+
+    key_a = store.compute_key(
+        route,
+        params,
+        cache_mod.CacheSettings(invariant_filters=(first, second), **base_settings),
+    )
+    key_b = store.compute_key(
+        route,
+        params,
+        cache_mod.CacheSettings(invariant_filters=(second, first), **base_settings),
+    )
+
+    assert key_a.digest == key_b.digest
+
+
 @pytest.mark.skipif(TestClient is None, reason="fastapi is not available")
 def test_invariant_unique_values_merge_with_static_options(tmp_path: Path) -> None:
     src = tmp_path / "src"
@@ -728,3 +921,4 @@ def test_invariant_unique_values_merge_with_static_options(tmp_path: Path) -> No
     assert "<option value='Engineering'>Engineering</option>" in text
     assert "<option value='Finance'>Finance</option>" in text
     assert "<option value='Other'>Custom</option>" in text
+from decimal import Decimal
